@@ -51,8 +51,6 @@ enum
 {
     /* Ready for dispatching */
     READY,
-    /* Approved by an approver, can only happens on incoming operations */
-    APPROVED,
     /* Claimed by a handler */
     CLAIMED,
     /* Error, channel went away, inspecting it failed etc */
@@ -70,6 +68,7 @@ enum {
   PROP_CONTACT,
   PROP_INCOMING,
   PROP_STATUS,
+  PROP_USER_ACTION_TIME,
 };
 
 /* private structure */
@@ -85,7 +84,7 @@ struct _EmpathyDispatchOperationPriv
   EmpathyContact *contact;
   EmpathyDispatchOperationState status;
   gboolean incoming;
-  gboolean approved;
+  gint64 user_action_time;
   gulong invalidated_handler;
   gulong ready_handler;
 };
@@ -132,6 +131,10 @@ empathy_dispatch_operation_set_property (GObject *object,
       case PROP_INCOMING:
         priv->incoming = g_value_get_boolean (value);
         break;
+
+      case PROP_USER_ACTION_TIME:
+        priv->user_action_time = g_value_get_int64 (value);
+        break;
     }
 }
 
@@ -162,6 +165,9 @@ empathy_dispatch_operation_get_property (GObject *object,
       case PROP_STATUS:
         g_value_set_enum (value, priv->status);
         break;
+      case PROP_USER_ACTION_TIME:
+        g_value_set_int64 (value, priv->user_action_time);
+        break;
     }
 }
 
@@ -176,7 +182,7 @@ empathy_dispatch_operation_invalidated (TpProxy *proxy, guint domain,
 }
 
 static void
-dispatcher_operation_got_contact_cb (EmpathyTpContactFactory *factory,
+dispatcher_operation_got_contact_cb (TpConnection *connection,
                                      EmpathyContact *contact,
                                      const GError *error,
                                      gpointer user_data,
@@ -210,8 +216,8 @@ dispatch_operation_connection_ready (TpConnection *connection,
 {
   EmpathyDispatchOperation *self = EMPATHY_DISPATCH_OPERATION (user_data);
   EmpathyDispatchOperationPriv *priv = GET_PRIV (self);
-  EmpathyTpContactFactory *factory;
   TpHandle handle;
+  TpHandleType handle_type;
 
   if (error != NULL)
     goto out;
@@ -220,14 +226,19 @@ dispatch_operation_connection_ready (TpConnection *connection,
     /* no point to get more information */
     goto out;
 
-  handle = tp_channel_get_handle (priv->channel, NULL);
+  handle = tp_channel_get_handle (priv->channel, &handle_type);
+  if (handle_type == TP_HANDLE_TYPE_CONTACT && priv->contact == NULL)
+    {
+      empathy_tp_contact_factory_get_from_handle (priv->connection, handle,
+        dispatcher_operation_got_contact_cb, NULL, NULL, G_OBJECT (self));
+    }
+  else
+    {
+      g_object_ref (self);
+      tp_channel_call_when_ready (priv->channel,
+          empathy_dispatch_operation_channel_ready_cb, self);
+    }
 
-  factory = empathy_tp_contact_factory_dup_singleton (priv->connection);
-
-  empathy_tp_contact_factory_get_from_handle (factory, handle,
-      dispatcher_operation_got_contact_cb, NULL, NULL, G_OBJECT (self));
-
-  g_object_unref (factory);
 out:
   g_object_unref (self);
 }
@@ -237,8 +248,6 @@ empathy_dispatch_operation_constructed (GObject *object)
 {
   EmpathyDispatchOperation *self = EMPATHY_DISPATCH_OPERATION (object);
   EmpathyDispatchOperationPriv *priv = GET_PRIV (self);
-  TpHandle handle;
-  TpHandleType handle_type;
 
   empathy_dispatch_operation_set_status (self,
     EMPATHY_DISPATCHER_OPERATION_STATE_PREPARING);
@@ -247,21 +256,9 @@ empathy_dispatch_operation_constructed (GObject *object)
     g_signal_connect (priv->channel, "invalidated",
       G_CALLBACK (empathy_dispatch_operation_invalidated), self);
 
-  handle = tp_channel_get_handle (priv->channel, &handle_type);
-
-  if (handle_type == TP_HANDLE_TYPE_CONTACT && priv->contact == NULL)
-    {
-      /* Ensure to keep the self object alive while the call_when_ready is
-       * running */
-      g_object_ref (self);
-      tp_connection_call_when_ready (priv->connection,
-          dispatch_operation_connection_ready, object);
-      return;
-    }
-
   g_object_ref (self);
-  tp_channel_call_when_ready (priv->channel,
-    empathy_dispatch_operation_channel_ready_cb, self);
+  tp_connection_call_when_ready (priv->connection,
+          dispatch_operation_connection_ready, object);
 }
 
 static void
@@ -290,14 +287,6 @@ empathy_dispatch_operation_class_init (
       g_cclosure_marshal_VOID__VOID,
       G_TYPE_NONE, 0);
 
-  signals[APPROVED] = g_signal_new ("approved",
-    G_OBJECT_CLASS_TYPE(empathy_dispatch_operation_class),
-      G_SIGNAL_RUN_LAST,
-      0,
-      NULL, NULL,
-      g_cclosure_marshal_VOID__VOID,
-      G_TYPE_NONE, 0);
-
   signals[CLAIMED] = g_signal_new ("claimed",
     G_OBJECT_CLASS_TYPE(empathy_dispatch_operation_class),
       G_SIGNAL_RUN_LAST,
@@ -357,6 +346,13 @@ empathy_dispatch_operation_class_init (
     EMPATHY_TYPE_DISPATCH_OPERATION_STATE, 0,
     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_STATUS, param_spec);
+
+  param_spec = g_param_spec_int64 ("user-action-time",
+    "user action time", "The user action time of the operation",
+    G_MININT64, G_MAXINT64, 0,
+    G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_USER_ACTION_TIME,
+      param_spec);
 }
 
 void
@@ -420,6 +416,18 @@ empathy_dispatch_operation_set_status (EmpathyDispatchOperation *self,
     }
 }
 
+static void
+channel_wrapper_ready (EmpathyDispatchOperation *self)
+{
+  EmpathyDispatchOperationPriv *priv = GET_PRIV (self);
+
+  g_signal_handler_disconnect (priv->channel_wrapper, priv->ready_handler);
+  priv->ready_handler = 0;
+
+  empathy_dispatch_operation_set_status (self,
+    EMPATHY_DISPATCHER_OPERATION_STATE_PENDING);
+}
+
 static void
 empathy_dispatcher_operation_tp_chat_ready_cb (GObject *object,
   GParamSpec *spec, gpointer user_data)
@@ -430,11 +438,7 @@ empathy_dispatcher_operation_tp_chat_ready_cb (GObject *object,
   if (!empathy_tp_chat_is_ready (EMPATHY_TP_CHAT (priv->channel_wrapper)))
     return;
 
-  g_signal_handler_disconnect (priv->channel_wrapper, priv->ready_handler);
-  priv->ready_handler = 0;
-
-  empathy_dispatch_operation_set_status (self,
-    EMPATHY_DISPATCHER_OPERATION_STATE_PENDING);
+  channel_wrapper_ready (self);
 }
 
 static void
@@ -473,11 +477,6 @@ empathy_dispatch_operation_channel_ready_cb (TpChannel *channel,
           goto out;
         }
     }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
-    {
-      EmpathyTpCall *call = empathy_tp_call_new (channel);
-      priv->channel_wrapper = G_OBJECT (call);
-    }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
     {
        EmpathyTpFile *file = empathy_tp_file_new (channel, priv->incoming);
@@ -492,29 +491,22 @@ out:
 }
 
 EmpathyDispatchOperation *
-empathy_dispatch_operation_new (TpConnection *connection, TpChannel *channel,
-  EmpathyContact *contact, gboolean incoming)
-{
-  return empathy_dispatch_operation_new_with_wrapper (connection, channel,
-    contact, incoming, NULL);
-}
-
-EmpathyDispatchOperation *
-empathy_dispatch_operation_new_with_wrapper (TpConnection *connection,
-  TpChannel *channel, EmpathyContact *contact, gboolean incoming,
-  GObject *wrapper)
+empathy_dispatch_operation_new (TpConnection *connection,
+    TpChannel *channel,
+    EmpathyContact *contact,
+    gboolean incoming,
+    gint64 user_action_time)
 {
   g_return_val_if_fail (connection != NULL, NULL);
   g_return_val_if_fail (channel != NULL, NULL);
 
-  return EMPATHY_DISPATCH_OPERATION (
-    g_object_new (EMPATHY_TYPE_DISPATCH_OPERATION,
+  return g_object_new (EMPATHY_TYPE_DISPATCH_OPERATION,
       "connection", connection,
       "channel", channel,
-      "channel-wrapper", wrapper,
       "contact", contact,
       "incoming", incoming,
-      NULL));
+      "user-action-time", user_action_time,
+      NULL);
 }
 
 void
@@ -529,47 +521,10 @@ empathy_dispatch_operation_start (EmpathyDispatchOperation *operation)
   g_return_if_fail (
     priv->status == EMPATHY_DISPATCHER_OPERATION_STATE_PENDING);
 
-  if (priv->incoming && !priv->approved)
-    empathy_dispatch_operation_set_status (operation,
-      EMPATHY_DISPATCHER_OPERATION_STATE_APPROVING);
-  else
-    empathy_dispatch_operation_set_status (operation,
+  empathy_dispatch_operation_set_status (operation,
       EMPATHY_DISPATCHER_OPERATION_STATE_DISPATCHING);
 }
 
-void
-empathy_dispatch_operation_approve (EmpathyDispatchOperation *operation)
-{
-  EmpathyDispatchOperationPriv *priv;
-
-  g_return_if_fail (EMPATHY_IS_DISPATCH_OPERATION (operation));
-
-  priv = GET_PRIV (operation);
-
-  if (priv->status == EMPATHY_DISPATCHER_OPERATION_STATE_APPROVING)
-    {
-      DEBUG ("Approving operation %s",
-        empathy_dispatch_operation_get_object_path (operation));
-
-      empathy_dispatch_operation_set_status (operation,
-        EMPATHY_DISPATCHER_OPERATION_STATE_DISPATCHING);
-
-      g_signal_emit (operation, signals[APPROVED], 0);
-    }
-  else if (priv->status < EMPATHY_DISPATCHER_OPERATION_STATE_APPROVING)
-    {
-      DEBUG ("Pre-approving operation %s",
-        empathy_dispatch_operation_get_object_path (operation));
-      priv->approved = TRUE;
-    }
-  else
-    {
-      DEBUG (
-        "Ignoring approval for %s as it's already past the approval stage",
-        empathy_dispatch_operation_get_object_path (operation));
-    }
-}
-
 /* Returns whether or not the operation was successfully claimed */
 gboolean
 empathy_dispatch_operation_claim (EmpathyDispatchOperation *operation)
@@ -692,3 +647,25 @@ empathy_dispatch_operation_is_incoming (EmpathyDispatchOperation *operation)
 
   return priv->incoming;
 }
+
+void
+empathy_dispatch_operation_set_user_action_time (
+    EmpathyDispatchOperation *self,
+    gint64 user_action_time)
+{
+  EmpathyDispatchOperationPriv *priv = GET_PRIV (self);
+
+  priv->user_action_time = user_action_time;
+}
+
+gint64
+empathy_dispatch_operation_get_user_action_time (EmpathyDispatchOperation *self)
+{
+  EmpathyDispatchOperationPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_DISPATCH_OPERATION (self), FALSE);
+
+  priv = GET_PRIV (self);
+
+  return priv->user_action_time;
+}
