@@ -110,7 +110,6 @@ enum
 enum
 {
   OBSERVE,
-  APPROVE,
   DISPATCH,
   LAST_SIGNAL
 };
@@ -430,16 +429,6 @@ dispatcher_channel_invalidated_cb (TpProxy *proxy,
   priv->channels = g_list_remove (priv->channels, proxy);
 }
 
-static void
-dispatch_operation_approved_cb (EmpathyDispatchOperation *operation,
-                                EmpathyDispatcher *self)
-{
-  g_assert (empathy_dispatch_operation_is_incoming (operation));
-  DEBUG ("Send of for dispatching: %s",
-    empathy_dispatch_operation_get_object_path (operation));
-  g_signal_emit (self, signals[DISPATCH], 0, operation);
-}
-
 static void
 dispatch_operation_claimed_cb (EmpathyDispatchOperation *operation,
                                EmpathyDispatcher *self)
@@ -480,9 +469,6 @@ dispatch_operation_ready_cb (EmpathyDispatchOperation *operation,
   ConnectionData *cd;
   EmpathyDispatchOperationState status;
 
-  g_signal_connect (operation, "approved",
-    G_CALLBACK (dispatch_operation_approved_cb), self);
-
   g_signal_connect (operation, "claimed",
     G_CALLBACK (dispatch_operation_claimed_cb), self);
 
@@ -508,19 +494,10 @@ dispatch_operation_ready_cb (EmpathyDispatchOperation *operation,
   if (status == EMPATHY_DISPATCHER_OPERATION_STATE_CLAIMED)
     return;
 
-  if (status == EMPATHY_DISPATCHER_OPERATION_STATE_APPROVING)
-    {
-      DEBUG ("Send to approvers: %s",
-        empathy_dispatch_operation_get_object_path (operation));
-      g_signal_emit (self, signals[APPROVE], 0, operation);
-    }
-  else
-    {
-      g_assert (status == EMPATHY_DISPATCHER_OPERATION_STATE_DISPATCHING);
-      DEBUG ("Send of for dispatching: %s",
-        empathy_dispatch_operation_get_object_path (operation));
-      g_signal_emit (self, signals[DISPATCH], 0, operation);
-    }
+  g_assert (status == EMPATHY_DISPATCHER_OPERATION_STATE_DISPATCHING);
+  DEBUG ("Send of for dispatching: %s",
+      empathy_dispatch_operation_get_object_path (operation));
+  g_signal_emit (self, signals[DISPATCH], 0, operation);
 
   g_object_unref (self);
 }
@@ -608,15 +585,6 @@ dispatcher_connection_new_channel (EmpathyDispatcher *self,
   DEBUG ("%s channel of type %s on %s", incoming ? "incoming" : "outgoing",
     channel_type, object_path);
 
-  if ((operation = g_hash_table_lookup (cd->dispatching_channels,
-      object_path)) != NULL)
-    {
-      /* This operation was already being dispatched, assume we got the channel
-       * again because something asked for it and approve it right away */
-      empathy_dispatch_operation_approve (operation);
-      return;
-    }
-
   if (properties == NULL)
     channel = tp_channel_new (connection, object_path, channel_type,
       handle_type, handle, NULL);
@@ -680,9 +648,6 @@ dispatcher_connection_new_channel (EmpathyDispatcher *self,
         }
     }
 
-  if (g_hash_table_lookup (cd->dispatched_channels, object_path) != NULL)
-    empathy_dispatch_operation_approve (operation);
-
   dispatcher_start_dispatching (dispatcher, operation, cd);
 }
 
@@ -796,27 +761,12 @@ dispatcher_connection_got_all (TpProxy *proxy,
     }
 }
 
-static void
-dispatcher_connection_advertise_capabilities_cb (TpConnection    *connection,
-                                                 const GPtrArray *capabilities,
-                                                 const GError    *error,
-                                                 gpointer         user_data,
-                                                 GObject         *self)
-{
-  if (error)
-    DEBUG ("Error: %s", error->message);
-}
-
 static void
 connection_ready_cb (TpConnection *connection,
     const GError *error,
     gpointer user_data)
 {
   EmpathyDispatcher *self = EMPATHY_DISPATCHER (user_data);
-  GPtrArray   *capabilities;
-  GType        cap_type;
-  GValue       cap = {0, };
-  const gchar *remove_ = NULL;
 
   if (error != NULL)
     {
@@ -828,29 +778,6 @@ connection_ready_cb (TpConnection *connection,
       TP_IFACE_CONNECTION_INTERFACE_REQUESTS, dispatcher_connection_got_all,
       NULL, NULL, G_OBJECT (self));
 
-  /* Advertise VoIP capabilities */
-  capabilities = g_ptr_array_sized_new (1);
-  cap_type = dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING,
-    G_TYPE_UINT, G_TYPE_INVALID);
-  g_value_init (&cap, cap_type);
-  g_value_take_boxed (&cap, dbus_g_type_specialized_construct (cap_type));
-  dbus_g_type_struct_set (&cap,
-        0, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-        1, TP_CHANNEL_MEDIA_CAPABILITY_AUDIO |
-           TP_CHANNEL_MEDIA_CAPABILITY_VIDEO |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_STUN  |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_GTALK_P2P |
-           TP_CHANNEL_MEDIA_CAPABILITY_NAT_TRAVERSAL_ICE_UDP,
-           G_MAXUINT);
-  g_ptr_array_add (capabilities, g_value_get_boxed (&cap));
-
-  tp_cli_connection_interface_capabilities_call_advertise_capabilities (
-    connection, -1, capabilities, &remove_,
-    dispatcher_connection_advertise_capabilities_cb,
-    NULL, NULL, G_OBJECT (self));
-
-  g_value_unset (&cap);
-  g_ptr_array_free (capabilities, TRUE);
 out:
   g_object_unref (self);
 }
@@ -1093,16 +1020,6 @@ empathy_dispatcher_class_init (EmpathyDispatcherClass *klass)
       G_TYPE_NONE,
       1, EMPATHY_TYPE_DISPATCH_OPERATION);
 
-  signals[APPROVE] =
-    g_signal_new ("approve",
-      G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST,
-      0,
-      NULL, NULL,
-      g_cclosure_marshal_VOID__OBJECT,
-      G_TYPE_NONE,
-      1, EMPATHY_TYPE_DISPATCH_OPERATION);
-
   signals[DISPATCH] =
     g_signal_new ("dispatch",
       G_TYPE_FROM_CLASS (klass),
