@@ -44,13 +44,8 @@
 #include <telepathy-glib/connection-manager.h>
 #include <telepathy-glib/interfaces.h>
 
-#ifdef ENABLE_TPL
 #include <telepathy-logger/log-manager.h>
-#include <telepathy-logger/log-store-empathy.h>
-#else
 
-#include <libempathy/empathy-log-manager.h>
-#endif /* ENABLE_TPL */
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-call-factory.h>
@@ -61,17 +56,17 @@
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-dispatch-operation.h>
 #include <libempathy/empathy-ft-factory.h>
+#include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-tp-chat.h>
 #include <libempathy/empathy-tp-call.h>
 
-#include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-location-manager.h>
 
 #include "empathy-main-window.h"
-#include "empathy-import-mc4-accounts.h"
 #include "empathy-accounts-common.h"
 #include "empathy-accounts-dialog.h"
+#include "empathy-chat-manager.h"
 #include "empathy-status-icon.h"
 #include "empathy-call-window.h"
 #include "empathy-chat-window.h"
@@ -132,17 +127,18 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
            * (a GtkNotebook) when we'll call empathy_chat_window_present_chat */
         }
 
-      empathy_chat_window_present_chat (chat);
+      empathy_chat_window_present_chat (chat,
+          empathy_dispatch_operation_get_user_action_time (operation));
+
+      if (empathy_tp_chat_is_invited (tp_chat, NULL))
+        {
+          /* We have been invited to the room. Add ourself as member as this
+           * channel has been approved. */
+          empathy_tp_chat_join (tp_chat);
+        }
 
       empathy_dispatch_operation_claim (operation);
     }
-  else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
-    {
-      EmpathyCallFactory *factory;
-
-      factory = empathy_call_factory_get ();
-      empathy_call_factory_claim_channel (factory, operation);
-    }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER)
     {
       EmpathyFTFactory *factory;
@@ -159,17 +155,14 @@ dispatch_cb (EmpathyDispatcher *dispatcher,
 }
 
 static void
-use_conn_notify_cb (EmpathyConf *conf,
+use_conn_notify_cb (GSettings *gsettings,
     const gchar *key,
     gpointer     user_data)
 {
   EmpathyConnectivity *connectivity = user_data;
-  gboolean     use_conn;
 
-  if (empathy_conf_get_bool (conf, key, &use_conn))
-    {
-      empathy_connectivity_set_use_conn (connectivity, use_conn);
-    }
+  empathy_connectivity_set_use_conn (connectivity,
+      g_settings_get_boolean (gsettings, key));
 }
 
 static void
@@ -344,8 +337,8 @@ account_manager_ready_cb (GObject *source_object,
   GError *error = NULL;
   EmpathyIdle *idle;
   EmpathyConnectivity *connectivity;
-  gboolean autoconnect = TRUE;
   TpConnectionPresenceType presence;
+  GSettings *gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
 
   if (!tp_account_manager_prepare_finish (manager, result, &error))
     {
@@ -361,23 +354,21 @@ account_manager_ready_cb (GObject *source_object,
   presence = tp_account_manager_get_most_available_presence (manager, NULL,
       NULL);
 
-  empathy_conf_get_bool (empathy_conf_get (),
-      EMPATHY_PREFS_AUTOCONNECT, &autoconnect);
-  if (autoconnect && !no_connect &&
+  if (g_settings_get_boolean (gsettings, EMPATHY_PREFS_AUTOCONNECT) &&
+      !no_connect &&
       tp_connection_presence_type_cmp_availability
           (presence, TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
             <= 0)
       /* if current state is Offline, then put it online */
       empathy_idle_set_state (idle, TP_CONNECTION_PRESENCE_TYPE_AVAILABLE);
 
-  /* Pop up the accounts dialog if it's needed (either when we don't have any
-   * accounts yet or when we haven't imported mc4 accounts yet */
-  if (!empathy_accounts_has_accounts (manager)
-      || !empathy_import_mc4_has_imported ())
+  /* Pop up the accounts dialog if we don't have any account */
+  if (!empathy_accounts_has_accounts (manager))
     show_accounts_ui (gdk_screen_get_default (), TRUE);
 
   g_object_unref (idle);
   g_object_unref (connectivity);
+  g_object_unref (gsettings);
 }
 
 static EmpathyDispatcher *
@@ -395,15 +386,9 @@ setup_dispatcher (void)
     { TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_ROOM  },
     /* file transfer to contacts */
     { TP_IFACE_CHANNEL_TYPE_FILE_TRANSFER, TP_HANDLE_TYPE_CONTACT  },
-    /* stream media to contacts */
-    { TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA, TP_HANDLE_TYPE_CONTACT  },
     /* roomlists */
     { TP_IFACE_CHANNEL_TYPE_ROOM_LIST, TP_HANDLE_TYPE_NONE },
   };
-  gchar *capabilities[] = {
-    "org.freedesktop.Telepathy.Channel.Interface.MediaSignalling/ice-udp",
-    "org.freedesktop.Telepathy.Channel.Interface.MediaSignalling/gtalk-p2p",
-    NULL };
   GHashTable *asv;
   guint i;
 
@@ -434,29 +419,8 @@ setup_dispatcher (void)
       g_ptr_array_add (filters, asv);
     }
 
-  asv = tp_asv_new (
-        TP_IFACE_CHANNEL ".ChannelType",
-          G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-        TP_IFACE_CHANNEL ".TargetHandleType",
-          G_TYPE_INT, TP_HANDLE_TYPE_CONTACT,
-        TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialAudio",
-          G_TYPE_BOOLEAN, TRUE,
-        NULL);
-  g_ptr_array_add (filters, asv);
-
-  asv = tp_asv_new (
-        TP_IFACE_CHANNEL ".ChannelType",
-          G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA,
-        TP_IFACE_CHANNEL ".TargetHandleType",
-          G_TYPE_INT, TP_HANDLE_TYPE_CONTACT,
-        TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialVideo",
-          G_TYPE_BOOLEAN, TRUE,
-        NULL);
-  g_ptr_array_add (filters, asv);
-
-
   empathy_dispatcher_add_handler (d, PACKAGE_NAME"MoreThanMeetsTheEye",
-    filters, capabilities);
+    filters, NULL);
 
   g_ptr_array_foreach (filters, (GFunc) g_hash_table_destroy, NULL);
   g_ptr_array_free (filters, TRUE);
@@ -480,7 +444,8 @@ account_status_changed_cb (TpAccount *account,
     return;
 
   empathy_dispatcher_join_muc (conn,
-      empathy_chatroom_get_room (room), NULL, NULL);
+      empathy_chatroom_get_room (room), EMPATHY_DISPATCHER_NON_USER_ACTION,
+      NULL, NULL);
 }
 
 static void
@@ -528,7 +493,8 @@ account_manager_chatroom_ready_cb (GObject *source_object,
           else
             {
               empathy_dispatcher_join_muc (conn,
-                  empathy_chatroom_get_room (room), NULL, NULL);
+                  empathy_chatroom_get_room (room),
+                  EMPATHY_DISPATCHER_NON_USER_ACTION, NULL, NULL);
             }
         }
 
@@ -549,6 +515,17 @@ chatroom_manager_ready_cb (EmpathyChatroomManager *chatroom_manager,
       account_manager_chatroom_ready_cb, chatroom_manager);
 }
 
+static void
+empathy_idle_set_auto_away_cb (GSettings *gsettings,
+				const gchar *key,
+				gpointer user_data)
+{
+	EmpathyIdle *idle = user_data;
+
+	empathy_idle_set_auto_away (idle,
+      g_settings_get_boolean (gsettings, key));
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -558,24 +535,22 @@ main (int argc, char *argv[])
   EmpathyStatusIcon *icon;
   EmpathyDispatcher *dispatcher;
   TpAccountManager *account_manager;
-#ifdef ENABLE_TPL
   TplLogManager *log_manager;
-#else
-  EmpathyLogManager *log_manager;
-#endif /* ENABLE_TPL */
   EmpathyChatroomManager *chatroom_manager;
   EmpathyCallFactory *call_factory;
   EmpathyFTFactory  *ft_factory;
   GtkWidget *window;
   EmpathyIdle *idle;
   EmpathyConnectivity *connectivity;
+  EmpathyChatManager *chat_manager;
   GError *error = NULL;
   UniqueApp *unique_app;
   gboolean chatroom_manager_ready;
-
+  gboolean autoaway = TRUE;
 #ifdef ENABLE_DEBUG
   TpDebugSender *debug_sender;
-#endif /* ENABLE_TPL */
+#endif
+  GSettings *gsettings;
 
   GOptionContext *optcontext;
   GOptionEntry options[] = {
@@ -641,14 +616,23 @@ main (int argc, char *argv[])
 
   /* Setting up Idle */
   idle = empathy_idle_dup_singleton ();
-  empathy_idle_set_auto_away (idle, TRUE);
+
+  gsettings = g_settings_new (EMPATHY_PREFS_SCHEMA);
+  autoaway = g_settings_get_boolean (gsettings, EMPATHY_PREFS_AUTOAWAY);
+
+  g_signal_connect (gsettings,
+      "changed::" EMPATHY_PREFS_AUTOAWAY,
+      G_CALLBACK (empathy_idle_set_auto_away_cb), idle);
+
+  empathy_idle_set_auto_away (idle, autoaway);
 
   /* Setting up Connectivity */
   connectivity = empathy_connectivity_dup_singleton ();
-  use_conn_notify_cb (empathy_conf_get (), EMPATHY_PREFS_USE_CONN,
+  use_conn_notify_cb (gsettings, EMPATHY_PREFS_USE_CONN,
       connectivity);
-  empathy_conf_notify_add (empathy_conf_get (), EMPATHY_PREFS_USE_CONN,
-      use_conn_notify_cb, connectivity);
+  g_signal_connect (gsettings,
+      "changed::" EMPATHY_PREFS_USE_CONN,
+      G_CALLBACK (use_conn_notify_cb), connectivity);
 
   /* account management */
   account_manager = tp_account_manager_dup ();
@@ -662,19 +646,18 @@ main (int argc, char *argv[])
   migrate_config_to_xdg_dir ();
 
   /* Setting up UI */
-  window = empathy_main_window_show ();
+  window = empathy_main_window_dup ();
+  gtk_widget_show (window);
   icon = empathy_status_icon_new (GTK_WINDOW (window), start_hidden);
 
+  /* Chat manager */
+  chat_manager = empathy_chat_manager_dup_singleton ();
+
   g_signal_connect (unique_app, "message-received",
       G_CALLBACK (unique_app_message_cb), window);
 
   /* Logging */
-#ifdef ENABLE_TPL
   log_manager = tpl_log_manager_dup_singleton ();
-#else
-  log_manager = empathy_log_manager_dup_singleton ();
-  empathy_log_manager_observe (log_manager, dispatcher);
-#endif
 
   chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
   empathy_chatroom_manager_observe (chatroom_manager, dispatcher);
@@ -714,6 +697,7 @@ main (int argc, char *argv[])
   g_object_unref (debug_sender);
 #endif
 
+  g_object_unref (chat_manager);
   g_object_unref (idle);
   g_object_unref (connectivity);
   g_object_unref (icon);
@@ -726,6 +710,7 @@ main (int argc, char *argv[])
 #endif
   g_object_unref (ft_factory);
   g_object_unref (unique_app);
+  g_object_unref (gsettings);
 
   notify_uninit ();
   xmlCleanupParser ();
